{% load static %}
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{% block title %}FileChest{% endblock %}</title>
    <link rel="stylesheet" href="{% static 'filechest/css/style.css' %}">
    {% block extra_head %}{% endblock %}
</head>
<body>
    {% block content %}{% endblock %}

    {% block extra_js %}{% endblock %}

    {% include "filechest/includes/_footer.html" %}
</body>
</html>
