"""
URL configuration for adhoc mode.

Does not include admin or authentication URLs.
"""

from django.urls import path, include
from django.conf.urls.static import static
from django.conf import settings

urlpatterns = [
    path('', include('filechest.urls')),
]
