<footer class="footer">
    <p>Copyright &copy; 2026 FileChest</p>
    <p><a href="https://github.com/atsuoishimoto/django-filechest" target="_blank">github.com/atsuoishimoto/django-filechest</a></p>
</footer>
